import unittest
import biomart
import os

class BiomartServerTestCase(unittest.TestCase):
    def setUp(self):
        self.server = biomart.BiomartServer( url = biomart.TEST_URL )
        self.server.http_proxy = os.environ.get('http_proxy', None)
    
    def testCanConnect(self):
        self.assertTrue( self.server.is_alive() )
    
    def testHasMartservice(self):
        self.assertTrue( '/martservice' in self.server.url )
    
    def testCanFetchDatabases(self):
        self.server.fetch_databases()
        self.assertTrue( len(self.server.databases.keys()) > 0 )
    
    def testCanShowDatabases(self):
        expected_databases = [
            'Eurexpress Biomart', 'Pancreas63', 'sequence', 'protists_variations_20', 'Prod_WHEAT', 
            'vb_sequence_mart_19', 'protists_mart_20', 'Sigenae_Oligo_Annotation_Ensembl_61', 'experiments', 
            'expression', 'WS220', 'Public_OBIOMART', 'Hsmm_Hmec', 'Prod_LMACULANSEDIT', 'ikmc', 
            'Prod_BOTRYTISEDIT', 'genomic_features', 'ontology', 'ENSEMBL_MART_ONTOLOGY', 'Prod_POPLAR_V2', 
            'Prod_', 'metazoa_mart_20', 'europhenomeannotations', 'GC_mart', 'Public_VITIS_12x', 
            'plants_variations_20', 'cildb_all_v2', 'vb_snp_mart_19', 'Prod_SCLEROEDIT', 
            'plants_mart_20', 'protists_sequence_mart_20', 'phytozome_mart', 'emma_biomart', 
            'ENSEMBL_MART_PLANT_SEQUENCE', 'htgt', 'fungi_mart_20', 'msd', 'EMAGE gene expression', 
            'Breast_mart_58', 'Sigenae Oligo Annotation (Ensembl 59)', 'functional_genomics', 'biomart', 
            'K562_Gm12878', 'cildb_inp_v2', 'biomartDB', 'vb_mart_19', 'CosmicMart', 'GRAMENE_MAP_38', 
            'ENSEMBL_MART_PLANT', 'metazoa_variations_20', 'Prod_POPLAR', 'snp', 'oncomodules', 'combinations', 
            'fungi_sequence_mart_20', 'HapMap_rel27', 'ENSEMBL_MART_PLANT_SNP', 'gmap_japonica', 
            'EMAGE browse repository', 'pride', 'Public_MAIZE', 'fungi_variations_20', 'metazoa_sequence_mart_20', 
            'Sigenae Oligo Annotation (Ensembl 56)', 'unimart', 'prod-intermart_1', 'biblioDB', 'GermOnline', 
            'vega', 'Public_TAIRV10', 'ensembl', 'EMAP anatomy ontology', 'plants_sequence_mart_20', 'REACTOME', 
            'Public_VITIS', 'QTL_MART', 'metazoa_genomic_features_mart_20', 'sequence_mart'
        ]
        self.assertTrue( expected_databases == self.server.show_databases())
    
    

def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(BiomartServerTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



class BiomartAttribute(object):
    def __init__(self, params):
        self.name = params['internalName']
        self.displayName = ('displayName' in params and params['displayName'] == 'true')
        self.default = ('default' in params and params['default'] == 'true')
        self.hidden = ('hidden' in params and params['hidden'] == 'true')
    
    def __repr__(self):
        if self.default:
            return "%s (default)" % self.displayName
        return self.name
    



import unittest
import biomart
import os
import requests

class BiomartDatasetTestCase(unittest.TestCase):
    def setUp(self):
        self.dataset = biomart.BiomartDataset( url = biomart.TEST_URL, params = { 'name': 'uniprot' })
        self.dataset.http_proxy = os.environ.get('http_proxy', None)
    
    def testCanConnectToDataset(self):
        self.assertTrue( self.dataset.is_alive() )
    
    def testCanFetchConfiguration(self):
        self.dataset.fetch_configuration()
        self.assertTrue( len(self.dataset.filters.keys()) > 0 and len(self.dataset.attributes.keys()) > 0 )
    
    def testCanShowFilters(self):
        filters = self.dataset.show_filters()
        self.assertTrue( len(filters) > 0 )
    
    def testCanShowAttributes(self):
        attributes = self.dataset.show_attributes()
        self.assertTrue( len(attributes) > 0 )
    
    def testCanSearchWithDefaultAttributes(self):
        response = self.dataset.search({
            'filters': { 'accession': 'Q9MY58' }
        })
        self.assertTrue( response.status_code == requests.codes.ok )
    
    def testCanSearchWithMultipleFilters(self):
        response = self.dataset.search({
            'filters': { 'accession': ['Q9MY58','Q9URL2','Q9HGK6'] }
        })
        rows = [line for line in response.iter_lines()]
        self.assertTrue( len(rows) == 3 )
    

def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(BiomartDatasetTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



import requests
from xml.etree.ElementTree import ElementTree, Element, SubElement, tostring, fromstring

import biomart

class BiomartDataset(biomart.BiomartServer):
    def __init__(self, url, params, *args, **kwargs):
        super(BiomartDataset, self).__init__(url, *args, **kwargs)
        
        if not 'name' in params:
            raise biomart.BiomartException( "'name' must be specified." )
        
        self.name = params['name']
        
        self.displayName = params.get('displayName', None)
        self.visible     = (params.get('visible', 0) == 1)
        self._filters    = {}
        self._attributes = {}
    
    def __repr__(self):
        return self.name
    
    @property
    def attributes(self):
        if not self._attributes:
            self.fetch_configuration()
        return self._attributes
    
    @property
    def filters(self):
        if not self._filters:
            self.fetch_configuration()
        return self._filters
    
    def show_filters(self):
        if not self._filters:
            self.fetch_configuration()
        return self._filters.keys()
    
    def show_attributes(self):
        if not self._attributes:
            self.fetch_configuration()
        return self._attributes.keys()
    
    def fetch_configuration(self):
        self.assert_alive()
        
        r = requests.get( self.url, params = { 'type': 'configuration', 'dataset': self.name }, proxies = self.proxies )
        xml = fromstring( r.text )
        
        # Filters
        for filter_description in xml.iter( 'FilterDescription' ):
            name = filter_description.attrib['internalName']
            self._filters[name] = biomart.BiomartFilter( filter_description.attrib )
        
        # Attributes
        for attribute_description in xml.iter( 'AttributeDescription' ):
            name = attribute_description.attrib['internalName']
            self._attributes[name] = biomart.BiomartAttribute( attribute_description.attrib )
    
    def count( self, params ):
        return self.search( params, count = True )
    
    def search( self, params = {}, header = 0, count = False ):
        self.assert_alive()
        
        if not self._filters or not self._attributes:
            self.fetch_configuration()
        
        root = Element( 'Query' )
        root.set('virtualSchemaName', 'default')
        root.set('formatter', 'TSV')
        root.set('header', str(header))
        root.set('uniqueRows', '1')
        root.set('datasetConfigVersion', '0.6')
        if count:
            root.set('count', '1')
        
        dataset = SubElement( root, "Dataset" )
        dataset.set( 'name', self.name )
        dataset.set( 'interface', 'default' )
        
        filters    = params.get( 'filters', {} )
        attributes = params.get( 'attributes', [] )
        
        # Add filters to XML
        if filters:
            for name, value in filters.items():
                try:
                    filter = self.filters[name]
                except KeyError:
                    raise biomart.BiomartException( "The filter '%s' does not exist. Use one of: " % (name, ', '.join(self.attributes.keys())) )
                
                filter_elem = SubElement( dataset, "Filter" )
                filter_elem.set( 'name', name )
                
                if filter.type == 'boolean':
                    if value == True or value.lower() == 'included' or value.lower() == 'only':
                        filter_elem.set( 'excluded', '0' )
                    elif value == False or value.lower() == 'excluded':
                        filter_elem.set( 'excluded', '1' )
                    else:
                        raise biomart.BiomartException( "The boolean filter '%s' can only accept True, 'included', 'only', False, 'excluded'" % self.name )
                
                else:
                    if isinstance( value, list ) or isinstance( value, tuple ):
                        value = ",".join( value )
                    filter_elem.set( 'value', value )
        else:
            for filter in self.filters.values():
                if filter.default and filter.default_value:
                    filter_elem = SubElement( dataset )
                    filter_elem.set( 'name', str(filter.name) )
                    if filter.type == 'boolean':
                        filter_elem.set( 'excluded', str(filter.default_value) )
                    else:
                        filter_elem.set( 'value', str(filter.default_value) )
        
        # Add attributes to XML, unless "count"
        if not count:
            if attributes:
                for attribute_name in attributes:
                    if not attribute_name in self.attributes.keys():
                        raise biomart.BiomartException( "The Attribute '%s' does not exist" % attribute_name )
            else:
                attributes = [ attr.name for attr in self.attributes.values() if attr.default ]
            
            if not attributes:
                raise biomart.BiomartException('No attributes selected, please select at least one')
            
            for attribute_name in attributes:
                attribute_elem = SubElement( dataset, "Attribute" )
                attribute_elem.set( 'name', str(attribute_name) )
        
        return requests.get( self.url, params = { 'query': tostring( root ) }, proxies = self.proxies, stream = True )



import unittest
import biomart
import os

class BiomartDatabaseTestCase(unittest.TestCase):
    def setUp(self):
        self.database = biomart.BiomartDatabase( url = biomart.TEST_URL, params = { 'name': 'unimart' })
        self.database.http_proxy = os.environ.get('http_proxy', None)
        
    def testCanConnectToDatabase(self):
        self.assertTrue( self.database.is_alive() )
    
    def testCanFetchDatasets(self):
        self.database.fetch_datasets()
        self.assertTrue( len(self.database.datasets.keys()) > 0 )
    
    def testCanSelectDataset(self):
        uniprot = self.database.datasets['uniprot']
        self.assertTrue( uniprot.is_alive() )
    

def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(BiomartDatabaseTestCase))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



from server import BiomartServer
from database import BiomartDatabase
from dataset import BiomartDataset
from filter import BiomartFilter
from attribute import BiomartAttribute

TEST_URL="http://www.biomart.org/biomart"

class BiomartException(Exception):
    pass



import requests
import biomart

class BiomartDatabase(biomart.BiomartServer):
    def __init__(self, url, params, *args, **kwargs):
        super(BiomartDatabase, self).__init__(url, *args, **kwargs)
        
        if not 'name' in params:
            raise biomart.BiomartException("'name' arg must be specified.")
        
        self.name        = params['name']
        self.displayName = params.get('displayName', None)
        self.visible     = (params.get('visible', 0) == 1)
        self._datasets   = {}
    
    def __repr__(self):
        return self.name
    
    @property
    def datasets(self):
        if not self._datasets:
            self.fetch_datasets()
        return self._datasets
    
    def fetch_datasets(self):
        self.assert_alive()
        
        r = requests.get( self.url, params = { 'type': 'datasets', 'mart': self.name }, proxies = self.proxies )
        for line in r.iter_lines():
            line = line.rstrip("\n").split("\t")
            if len(line) > 3:
                params = {
                    'name': line[1],
                    'displayName': line[2],
                    'visible': int(line[3])
                }
                self._datasets[params['name']] = biomart.BiomartDataset( self.url, params, http_proxy = self.http_proxy )
        
    



import requests
from xml.etree.ElementTree import fromstring
import biomart

class BiomartServer(object):
    def __init__(self, url, http_proxy = '', https_proxy = ''):
        if not 'martservice' in url:
            url += '/martservice'
        if not 'http://' in url:
            url = 'http://' + url
        
        self.url = url
        self.proxies = { 'http': http_proxy, 'https': https_proxy }
        
        self._databases = {}
        self._datasets = {}
    
    def get_http_proxy(self):
        return self.proxies['http']
    def set_http_proxy(self, http_proxy):
        self.proxies['http'] = http_proxy
    http_proxy = property(get_http_proxy, set_http_proxy)
    
    def get_https_proxy(self):
        return self.proxies['https']
    def set_https_proxy(self, https_proxy):
        self.proxies['https'] = https_proxy
    https_proxy = property(get_https_proxy, set_https_proxy)
    
    def is_alive(self):
        r = requests.get( self.url, proxies = self.proxies )
        return r.status_code == requests.codes.ok
    
    def assert_alive(self):
        if not self.is_alive():
            raise biomart.BiomartException( "Server is not alive or could not be reached." )
    
    @property
    def databases(self):
        if not self._databases:
            self.fetch_databases()
        return self._databases
    
    @property
    def datasets(self):
        if not self._datasets:
            self.fetch_datasets()
        return self._datasets
    
    def fetch_databases(self):
        self.assert_alive()
        
        r = requests.get( self.url, params = { 'type': 'registry' }, proxies = self.proxies )
        xml = fromstring(r.text)
        for child in xml:
            if child.tag == 'MartURLLocation':
                database = biomart.BiomartDatabase(
                    url         = self.url, 
                    params      = child.attrib, 
                    http_proxy  = self.http_proxy, 
                    https_proxy = self.https_proxy
                )
                self._databases[ child.attrib['name'] ] = database
    
    def fetch_datasets(self):
        self.assert_alive()
        
        for database in self.databases.values():
            self._datasets.update( database.datasets )
    
    def show_databases(self):
        if not self._databases:
            self.fetch_databases()
        
        return self._databases.keys()
    
    def show_datasets(self):
        if not self._datasets:
            self.fetch_datasets()
        
        return self._datasets.keys()
    
